==> cloudt-boot-starter-datasecurity-0.2.12-20240319.081349-1-sources.jar: com/elitescloud/boot/datasecurity/jpa/strategy/RoleRuleJpaService.java <==
package com.elitescloud.boot.datasecurity.jpa.strategy;

import cn.hutool.core.collection.CollUtil;
import com.elitescloud.boot.datasecurity.dpr.content.DprRuleRelationEnum;
import com.elitescloud.boot.datasecurity.dpr.service.RoleDataPermissionRuleService;
import com.elitescloud.boot.datasecurity.dpr.service.util.RoleDatePermissionColumnUtil;
import com.elitescloud.cloudt.system.dto.SysDprRoleApiDataRuleListQueryDTO;
import com.elitescloud.cloudt.system.dto.SysDprRoleApiRowColumnRuleDTO;
import com.querydsl.core.BooleanBuilder;
import com.querydsl.core.types.Predicate;
import com.querydsl.core.types.dsl.PathBuilder;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author : chen.niu
 * @description : 数据权限JPA应用的接口
 * @date : 2023/10/14 18:46
 */
@Slf4j
public class RoleRuleJpaService {
    private final RoleDataPermissionRuleService roleDataPermissionRuleService;

    public RoleRuleJpaService(RoleDataPermissionRuleService roleDataPermissionRuleService) {
        this.roleDataPermissionRuleService = roleDataPermissionRuleService;
    }

    /***
     * 获取当前用户的全部角色数据权限配置
     * **/
    public SysDprRoleApiRowColumnRuleDTO getUserRoleDpr() {
        return roleDataPermissionRuleService.getUserRoleDpr();
    }

    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     * 如果菜单编码为空将会抛异常，强制要求前端传递过来。
     * @param entityClass jpa实体类
     * @param alias jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     * @deprecated  异常抛出无捕获
     * **/
    public  Predicate getUserRoleApiMenusAuthJpaPredicate(Class entityClass, String alias){
        var predicate = getUserRoleApiMenusAuthJpaPredicate(entityClass,alias,null);
        log.info("数据权限过滤条件：{}", predicate == null ? null : predicate.toString());
        return predicate;
    }

    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     * 如果菜单编码为空将会抛异常，强制要求前端传递过来。
     * @param entityClass jpa实体类
     * @param alias jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     * @param ruleFieldName 可以null，需要指定规则字段配料的字段字符串返回规则。  特殊情况使用，如：只需要生成某个字段规则的条件，其他的不需要。
     * **/
    private Predicate getUserRoleApiMenusAuthJpaPredicate(Class entityClass, String alias, String[] ruleFieldName) {
        var userRoleDpr = roleDataPermissionRuleService.getUserRoleDpr();
        if(userRoleDpr==null || CollUtil.isEmpty(userRoleDpr.getSysDprRoleApiDataRuleListQueryDTO())){
            log.info("没有找到当前用户数据权限配置-跳过数据权限");
            return new BooleanBuilder(null);
        }
        //行数据权限配置 根据请求的path路径  比对当前用户全部的api行数据权限配置。 找到对应的api数据权限配置规则组和规则信息并返回
        List<SysDprRoleApiDataRuleListQueryDTO> startDto =
                RoleDatePermissionColumnUtil.apiPathRoleApiRowsRuleGroupDtoHandle(null, userRoleDpr);
        if(ruleFieldName!=null&&ruleFieldName.length>0){
            startDto = startDto.stream()
                    .filter(dpr -> Arrays.asList(ruleFieldName).contains(dpr.getDprRuleField()))
                    .collect(Collectors.toList());
        }
        if(CollUtil.isEmpty(startDto)){
            log.info("没有找到当前用户数据权限配置匹配规则-跳过数据权限");
            return new BooleanBuilder(null);
        }

        //queryDsl动态构造
        PathBuilder entityPath = new PathBuilder<>(entityClass, alias);
        BooleanBuilder returnPredicate = new BooleanBuilder();
        List<Predicate> groupPredicates = new ArrayList<Predicate>();
        if (startDto != null && !startDto.isEmpty()) {
            var groupMap=  groupByRole(startDto);
            //根据角色分组，每个角色直接的条件采用(（角色1条件）or（角色2条件）)方式
            for (Map.Entry<Long, List<SysDprRoleApiDataRuleListQueryDTO>> entry : groupMap.entrySet()) {
                List<SysDprRoleApiDataRuleListQueryDTO> dtoList = entry.getValue();
                BooleanBuilder groupPredicate = new BooleanBuilder();
                for (SysDprRoleApiDataRuleListQueryDTO sysRule : dtoList) {
                    try {
                        var predicateRule = RuleStrategyManager.getPathAutomatically(entityPath, sysRule);
                        //单个角色内的or或and条件
                        if(DprRuleRelationEnum.DPR_RULE_RELATION_AND.name().equals(sysRule.getDprRuleRelation())){
                            groupPredicate.and(predicateRule);
                        } else if(DprRuleRelationEnum.DPR_RULE_RELATION_OR.name().equals(sysRule.getDprRuleRelation())){
                            groupPredicate.or(predicateRule);
                        }else{
                            log.error("规则的关系异常:{}",sysRule);
                        }

                    }catch (Exception e){
                        log.error("规则字段匹配异常：",e);
                    }
                }
                if (groupPredicate.hasValue()) {
                    //不同角色的先存在List中
                    groupPredicates.add( groupPredicate.getValue());
                }

            }
            Predicate[] integersArray2 = groupPredicates.stream().toArray(Predicate[]::new);
            return returnPredicate.andAnyOf(integersArray2);
        }
        return new BooleanBuilder(null);
    }
    public Map<Long, List<SysDprRoleApiDataRuleListQueryDTO>> groupByRole(List<SysDprRoleApiDataRuleListQueryDTO> startDto) {
        return startDto.stream()
                .collect(Collectors.groupingBy(SysDprRoleApiDataRuleListQueryDTO::getRoleId));
    }
    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     *  这个方法菜单编码可以为空，如果空跳过权限。主要用于不好判断是否一定前端请求的接口，有的接口可能前端数据权限调用或采用定时任务或MQ消费复用的情况。
     * @param entityClass jpa实体类
     * @param alias jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     * **/
    public Predicate getUserRoleApiMenusAuthJpaPredicateMenusCodeEmpty(Class<?> entityClass, String alias) {
        String menuCode="";
        try {
              menuCode = RoleDatePermissionColumnUtil.verificationMenu();
            if (menuCode == null) {
                log.info("数据权限需要菜单编码将跳过数据权限，getHeader  menuCode=NULL");
                return new BooleanBuilder(null);
            } else {
                return getUserRoleApiMenusAuthJpaPredicate(entityClass, alias);
            }
        } catch (Exception e) {
            StringBuffer sb = new StringBuffer();
            sb.append(entityClass.getSimpleName()).append("-").append(menuCode).append("-").append(alias);
            log.error("JPA权限异常，异常需要分析，暂跳过权限功能："+sb, e);
            return new BooleanBuilder(null);
        }
    }

    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     *  这个方法菜单编码可以为空，如果空跳过权限。主要用于不好判断是否一定前端请求的接口，有的接口可能前端数据权限调用或采用定时任务或MQ消费复用的情况。
     * jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     *
     * @param entityClass jpa实体类
     * @
     * **/
    public Predicate getUserRoleApiMenusAuthJpaPredicateMenusCodeEmpty(Class<?> entityClass) {
        String menuCode="";
        try {
            menuCode= RoleDatePermissionColumnUtil.verificationMenu();
            if (menuCode == null) {
                log.info("数据权限需要菜单编码将跳过数据权限，getHeader  menuCode=NULL");
                return new BooleanBuilder(null);
            } else {
                //如果没有别名，使用JPA默认规范，DO Class名称，首字母小写。
                var classAlias=entityClass.getSimpleName().substring(0, 1).toLowerCase()
                        + entityClass.getSimpleName().substring(1);
                return getUserRoleApiMenusAuthJpaPredicate(entityClass, classAlias);
            }
        } catch (Exception e) {
            StringBuffer sb = new StringBuffer();
            sb.append(entityClass.getSimpleName()).append("-").append(menuCode).append("-");
            log.error("JPA权限异常，异常需要分析，暂跳过权限功能："+sb, e);
            return new BooleanBuilder(null);
        }
    }


    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     *  这个方法菜单编码可以为空，如果空跳过权限。主要用于不好判断是否一定前端请求的接口，有的接口可能前端数据权限调用或采用定时任务或MQ消费复用的情况。
     * jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     *
     * @param entityClass jpa实体类
     * @
     * **/
    public Predicate getUserRoleApiMenusAuthJpaPredicateMenusCodeEmpty(Class<?> entityClass, String alias,String... ruleFieldName) {
        String menuCode="";
        try {
            menuCode= RoleDatePermissionColumnUtil.verificationMenu();
            if (menuCode == null) {
                log.info("数据权限需要菜单编码将跳过数据权限，getHeader  menuCode=NULL");
                return new BooleanBuilder(null);
            } else {
                return getUserRoleApiMenusAuthJpaPredicate(entityClass, alias,ruleFieldName);
            }
        } catch (Exception e) {
            StringBuffer sb = new StringBuffer();
            sb.append(entityClass.getSimpleName()).append("-").append(menuCode).append("-");
            log.error("JPA权限异常，异常需要分析，暂跳过权限功能："+sb, e);
            return new BooleanBuilder(null);
        }
    }


}

==> cloudt-boot-starter-datasecurity-3.2.5.78.2-20231124.053433-1-sources.jar: com/elitescloud/boot/datasecurity/jpa/strategy/RoleRuleJpaService.java <==
package com.elitescloud.boot.datasecurity.jpa.strategy;

import cn.hutool.core.collection.CollUtil;
import com.elitescloud.boot.datasecurity.common.DataSecurityConstant;
import com.elitescloud.boot.datasecurity.dpr.service.RoleDataPermissionRuleService;
import com.elitescloud.boot.datasecurity.dpr.service.util.RoleDatePermissionColumnUtil;
import com.elitescloud.cloudt.context.util.HttpServletUtil;
import com.elitescloud.cloudt.system.dto.SysDprRoleApiDataRuleListQueryDTO;
import com.elitescloud.cloudt.system.dto.SysDprRoleApiRowColumnRuleDTO;
import com.querydsl.core.BooleanBuilder;
import com.querydsl.core.types.Predicate;
import com.querydsl.core.types.dsl.Expressions;
import com.querydsl.core.types.dsl.PathBuilder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author : chen.niu
 * @description : 数据权限JPA应用的接口
 * @date : 2023/10/14 18:46
 */
@Slf4j
public class RoleRuleJpaService {
    private final RoleDataPermissionRuleService roleDataPermissionRuleService;

    public RoleRuleJpaService(RoleDataPermissionRuleService roleDataPermissionRuleService) {
        this.roleDataPermissionRuleService = roleDataPermissionRuleService;
    }

    /***
     * 获取当前用户的全部角色数据权限配置
     * **/
    public SysDprRoleApiRowColumnRuleDTO getUserRoleDpr() {
        return roleDataPermissionRuleService.getUserRoleDpr();
    }


    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     * 如果菜单编码为空将会抛异常，强制要求前端传递过来。
     * @param entityClass jpa实体类
     * @param alias jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     * **/
    public Predicate getUserRoleApiMenusAuthJpaPredicate(Class entityClass, String alias) {
        var userRoleDpr = roleDataPermissionRuleService.getUserRoleDpr();
        if(userRoleDpr==null || CollUtil.isEmpty(userRoleDpr.getSysDprRoleApiDataRuleListQueryDTO())){
            log.warn("没有找到当前用户数据权限配置-跳过数据权限");
            return new BooleanBuilder(null);
        }
        //行数据权限配置 根据请求的path路径  比对当前用户全部的api行数据权限配置。 找到对应的api数据权限配置规则组和规则信息并返回
        List<SysDprRoleApiDataRuleListQueryDTO> startDto =
                RoleDatePermissionColumnUtil.apiPathRoleApiRowsRuleGroupDtoHandle(null, userRoleDpr);
        if(CollUtil.isEmpty(startDto)){
            log.warn("没有找到当前用户数据权限配置匹配规则-跳过数据权限");
            return new BooleanBuilder(null);
        }

        //queryDsl动态构造
        PathBuilder entityPath = new PathBuilder<>(entityClass, alias);
        if (startDto != null && !startDto.isEmpty()) {
            BooleanBuilder group1 = new BooleanBuilder();
            for (SysDprRoleApiDataRuleListQueryDTO sysRule : startDto) {
                var predicateRule = RuleStrategyManager.getPathAutomatically(entityPath, sysRule);
//                predicate.ann;
                group1.and(predicateRule);
            }
            return group1;
        }
        return new BooleanBuilder(null);
    }

    /***
     * 返回当前用户的访问的API的菜单的角色权限的JPA Predicate条件
     *  这个方法菜单编码可以为空，如果空跳过权限。主要用于不好判断是否一定前端请求的接口，有的接口可能前端数据权限调用或采用定时任务或MQ消费复用的情况。
     * @param entityClass jpa实体类
     * @param alias jpa sql查询表的别名，如果没有其就是jpa实体类的首字母小写名称 如SysUser 应该是sysUser
     * **/
    public Predicate getUserRoleApiMenusAuthJpaPredicateMenusCodeEmpty(Class<?> entityClass, String alias) {
        try {
            String menuCode = RoleDatePermissionColumnUtil.verificationMenu();
            if (menuCode == null) {
                log.warn("数据权限需要菜单编码将跳过数据权限，getHeader  menuCode=NULL");
                return new BooleanBuilder(null);
            } else {
                return getUserRoleApiMenusAuthJpaPredicate(entityClass, alias);
            }
        } catch (Exception e) {
            log.error("JPA权限异常，异常需要分析，暂跳过权限功能：" + e.getMessage());
            return new BooleanBuilder(null);
        }

    }

}
